#!/python/bin

import astropy.io.fits as py
from pylab import *
import matplotlib.pyplot as plt
from datetime import datetime as dt
import calendar
import astrodate
import time
import glob
import os
import pysynphot as S
from matplotlib.backends.backend_pdf import PdfPages

def UseafterToday():
    ''' Takes the current day ad uses it to fill the useafter_var
    '''
    months=dict((k,v) for k,v in enumerate(calendar.month_abbr))
    localtime = time.localtime(time.time())
    return  months[localtime.tm_mon] + ' ' + str(localtime.tm_mday) + ' ' + str(localtime.tm_year)


def toYearFraction(date):
    def sinceEpoch(date): # returns seconds since epoch
        return time.mktime(date.timetuple())
    s = sinceEpoch

    year = date.year
    startOfThisYear = dt(year=year, month=1, day=1)
    startOfNextYear = dt(year=year+1, month=1, day=1)

    yearElapsed = s(date) - s(startOfThisYear)
    yearDuration = s(startOfNextYear) - s(startOfThisYear)
    fraction = yearElapsed/yearDuration

    return date.year + fraction

def mjdDate(date):
    s_date = date.split()
    date=''
    if len(s_date) > 2:
        for s in xrange(2): date += s_date[s] + ' '
        date += s_date[2]
    ua_time=time.strptime(date, "%b %d %Y")
    ua_date = datetime.datetime(ua_time.tm_year,ua_time.tm_mon, ua_time.tm_mday)
    year_frac =toYearFraction(ua_date)
    return astrodate.JulianDate(year_frac).mjd


def dateNum(useafter_date):
    '''
    Acepts and string in the form Month day year hh:mm:ss where Month is
    the abreviated form of the monh name. It converts this sring to a
    dictioary with  year, month, day, hour, minute, seconds
    '''
    months=dict((k,v) for k,v in enumerate(calendar.month_abbr))

    month_1, day_1, year_1 = useafter_date.split


rootdir = '/usr/stsci/stdata/comp/wfc3/'
rootname = 'wfc3_'
figrootname = 'wfc3_fig_'

files = glob.glob(rootdir+rootname+'*.fits')
filetype='g'

# find the filter name of the file
l1 = files[0].find('_'+filetype)
if l1 > 0:
    wavfile0 = filetype+files[0][l1+2:l1+6]
else :  wavfile0 = ''
subfiles=[]
for i in xrange(len(files)):
    #for i in xrange(10):

    #assumes the files are in alphabetical order
    ft='_'+filetype
    l1 = files[i].find(ft)
    fq = files[i][l1+2:l1+3]

    # if it found the pattern and the next letter is q then account that on the
    # name
    if fq == 'q' and l1 != -1:
        wavfile = filetype + 'q'+files[i][l1+3:l1+7]
    elif fq != 'q' and l1 != -1: # if the second value is not q account for that
        wavfile = filetype +files[i][l1+2:l1+6]
    elif l1 == -1: # if didn't find the value then assing a dummy value
        wavfile = '-'


    #if the files have equal wavelength, group them
    if wavfile ==  wavfile0 and l1 > 0:
        subfiles.append(files[i])
        files[i]
    elif wavfile0 == '':
        l1 = files[i].find('_'+filetype)
        if l1 > 0:
            wavfile0 = filetype+files[i][l1+2:l1+6]
            subfiles.append(files[i])
        else :  wavfile0 = ''
    elif l1 > 0:
        l1 = files[i].find('_'+filetype)
        # Define name of output figure and put multiple PDF figures there
        figurename = figrootname + files[i][l1-4:l1] + '_' + wavfile + '.pdf'
        pp = PdfPages(figurename)

        wavfile0 = wavfile

        # Find the fles that have extension f1 and ignore them
        subsubfiles=[]
        for j in xrange(len(subfiles)):
             fl2 = subfiles[j].find('f1_')
             fl3 = subfiles[j].find('f2_')
             fl4 = subfiles[j].find('_bkg_')
             fl5 = subfiles[j].find('_th.fits')
             if logical_or.reduce([(fl2 != -1),(fl3 != -1),(fl4 != -1), (fl5 != -1)]):
                print 'Rejected =', subfiles[j]
                pass
             else:
                subsubfiles.append(subfiles[j])
        for h in xrange(len(subsubfiles)): print subsubfiles[h]

        # initialize tuples and varibles and arrays for wavelength and
        # throughput
        wmin0=1000.0
        wmax0=0.0
        wlen=[]
        nrowslen=[]
        mjd=[]
        useafter_in=[]
        ww = np.array([],dtype = float64)
        tt = np.array([],dtype = float64)

        plt.clf()
        for j in xrange(len(subsubfiles)):
            filein = py.open(subsubfiles[j])   # open file
            data = filein[1].data       # read data
            # get min an max wavlength values
            wmin=np.log10(np.amin(data.field('WAVELENGTH')))
            wmax=np.log10(np.amax(data.field('WAVELENGTH')))

            # Gets min and max wavelengths for all the availale files
            #of the same  filter
            if wmin < wmin0:
                 wmin0 = wmin
            if wmax > wmax0:
                 wmax0 = wmax

            # saves the original size of the array
            wlen.append(len(data.field('WAVELENGTH')))

            # Appends all the wavelenghts and throughputs in a single file
            ww = np.append(ww, np.array(data.field('WAVELENGTH'), dtype=float64), axis = 0)
            tt = np.append(tt, np.array(data.field('THROUGHPUT'), dtype=float64), axis = 0)

            # takes only one of the wavelength ranges, the last. This weill be
            # used, together with the  throughput of the last file as input for
            # the first version of this files.
            ww_p = np.array(data.field('WAVELENGTH'), dtype=float64)
            tt_p = np.array(data.field('THROUGHPUT'), dtype=float64)

            #number of rows per input file
            nrowslen.append(data.shape[0])

            #extract useafter date from fies and use it to calculate the  MJD
            # for the  column names of the mjd file.
            ua=filein[0].header['useafter']
            useafter_in.append(ua)

            ua_mjd = mjdDate(ua)
            mjd.append(ua_mjd)
            # extracts other useful keywrds
            dbtable = filein[0].header['dbtable']
            descrip = filein[0].header['descrip']
            compname= filein[0].header['compname']
            pedigree = filein[0].header['pedigree']
            instrument =filein[0].header['instrume']
            prihdr = filein[0].header

            filein.close()

        # define output filename
        l4 = subsubfiles[j].find('_0')
        l5 = subsubfiles[j].rfind('/')
        file_ver = int(subsubfiles[j][l4+1:l4+4])
        file_ver += 1
        outfile = subsubfiles[j][l5+1:l4+1] + 'mjd_00' + str.lstrip(str(file_ver)) + '_syn.fits'
        #outfile = subsubfiles[j][l5+1:l4+1] + '00' + str.lstrip(str(file_ver)) + '_syn.fits'
        if os.access(outfile, os.R_OK):
            os.remove(outfile)

        print 'outfile =',outfile

        # Define primary heder and add the needed keywords
        compname += '_m'
        newhdu= py.PrimaryHDU()
        newhdu.header.update('DBTABLE',dbtable, 'Synphot database name')
        newhdu.header.update('EXTRAP',True,'True: linear extrapolation')
        newhdu.header.update('DESCRIP',descrip)
        newhdu.header.update('COMPNAME',compname)
        newhdu.header.update('USEAFTER',UseafterToday())
        newhdu.header.update('PEDIGREE',pedigree)
        newhdu.header.update('INSTRUME',instrument)
        newhdu.header.update('COMMENT ','= Reference file created by R.Diaz', after='instrume')
        # Copies the history from the latest file and adds new input
        for  l in xrange(len(prihdr)-15):
            newhdu.header.add_history(prihdr[l+13])
        newhdu.header.add_history('  Previos file '+filein[0].header['comment'])
        newhdu.header.add_history(' ')
        newhdu.header.add_history('  Updated on ' + UseafterToday())
        newhdu.header.add_history(' ')
        newhdu.header.add_history('  New columns to suppor MJD calculations were added these are for')
        str_mjd=''
        for l in xrange(len(mjd)):
            str_mjd = str(useafter_in[l]) + '=' + str("%.2f" %
                    mjd[l]) + ';'
            newhdu.header.add_history('MJDs:' + str_mjd)
        newhdu.header.add_history('but the data is the same and equal to MJD'
                 + str("%.2f" % mjd[len(mjd)-1]))
        newhdu.header.add_history(' ')

        # Defines columns and asign their values
        col1= py.Column(name='WAVELENGTH', format='E', array=ww_p,
                unit='ANGSTROMS', disp='G15.7')
        col2= py.Column(name='THROUGHPUT', format='E', array=tt_p,
                unit='TRANSMISSION', disp='G15.7')
        cols0 = [col1,col2]

        #fo now it puts the same information for all the MJDs
        #in this case these thrghput corresponds to the newest file
        name_col = 'MJD#' + str("%.2f" % mjd[0])
        col3= py.Column(name=name_col, format='E', array=tt_p,
                unit='TRANSMISSION', disp='G15.7')
        cols0 = [col1,col2,col3]
        if len(subsubfiles) >= 2 :
            name_col = 'MJD#' + str("%.2f" % mjd[1])
            col4= py.Column(name=name_col, format='E', array=tt_p,
                    unit='TRANSMISSION', disp='G15.7')
            cols0.append(col4)
        if len(subsubfiles) >= 3:
            name_col = 'MJD#' + str("%.2f" % mjd[2])
            col5= py.Column(name=name_col, format='E',
                    array=tt_p, unit='TRANSMISSION', disp='G15.7')
            cols0.append(col5)
        if len(subsubfiles) >= 4:
            name_col = 'MJD#' + str("%.2f" % mjd[3])
            col6= py.Column(name=name_col, format='E', array=tt_p,
                    unit='TRANSMISSION', disp='G15.7')
            cols0.append(col6)

        cols = py.ColDefs(cols0, tbtype='TableHDU')
        tbhdu = py.new_table(cols)

        thdulist = py.HDUList([newhdu,tbhdu])
        thdulist.writeto(outfile)

        #ww1=np.reshape(ww,[len(wlen),wlen[0]])
        #tt1=np.reshape(tt,[len(wlen),wlen[0]])

        # Sorts wavelength arrays and leaves only unique values
        #ww1=np.sort(ww)
        #allwave = np.unique(ww1)
        # Makes a wavelength array where the wavelength is spaced evenly on a log
        # scale.

        #Total of 5000 points starting from 10 and ending on 1e7
        allwave=np.logspace(wmin0,wmax0,num=500,endpoint=True,base=10.0)

        fig1=plt.figure()
        ax1 = fig1.add_subplot(2,1,1)
        #subplot(2,1,1)
        tt3=[]
        nfiles = len(subsubfiles)
        plt.title('Filter '+ wavfile)
        nelements = 0
        for k in xrange(len(subsubfiles)):
            #sp = S.ArrayBandpass(wave=ww1[k], throughput=tt1[k], waveunits='angstroms')
            sp = S.ArrayBandpass(wave=ww[nelements:nelements+wlen[k]],
                    throughput=tt[nelements:nelements+wlen[k]], waveunits='angstroms')
            tt2= sp.sample(allwave)
            tt3.append(tt2)
            nelements += wlen[k]

            plt.semilogy()

            ax1.plot(allwave,tt2)

        #plt.gca().set_aspect('equal')

        #subplot(2,1,2)
        ax2 = fig1.add_subplot(212, sharex=ax1)
        plt.setp(ax1.get_xticklabels(), visible=False)
        plt.title('rate 2/1, 3/2, etc.')
        for k in xrange(len(subsubfiles)-1):
            nonzero = np.where(tt3[k] > 0)
            rate = tt3[k+1][nonzero]/tt3[k][nonzero]
            difference = (tt3[k+1][nonzero]-tt3[k][nonzero])/tt3[k][nonzero]

            plt.semilogy()
            #axis([10**wmin0,10**wmax0, 0.,5])
            ax2=plot(allwave[nonzero],rate)
        pp.savefig()

        plt.figure()
        plt.clf()
        #plt.axis()
        nonzero, = np.where(tt3[0] > 0.0)
        ymin=np.nanmin(tt3[0][nonzero])
        ymax=np.nanmax(tt3[0][nonzero])
        plt.title('Wavelength as function of timei ymin = ' + str(ymin))
        plt.xlim((0,3.2))
        plt.ylim((ymin,ymax))
        #plt.ylim((0.4,1.0))
        #plt.yscale('linear')
        #plt.semilogy()
        #locs, labels = plt.yticks()
        #plt.yticks( arange(6))
        #plt.show()
        #print locs, labels
        #plt.ticklabel_format(style='plain', axis='y')
        #plt.setp(ax1.get_yticklabels(), fontsize=6)
        for k in xrange(len(allwave)):
            newtt3=[]
            for  l in range(nfiles):
                newtt3.append(tt3[l][k])

            xscale('linear')
            yscale('linear')
            plot(range(nfiles),newtt3)
            plt.yscale('linear')
        #plt.show()

        if len(subsubfiles) > 1:
            rate = tt3[1][nonzero]/tt3[0][nonzero]
            plot(allwave[nonzero],rate)
        if len(subsubfiles) > 2:
            rate = tt3[0][nonzero]/tt3[2][nonzero]
            plt.semilogy()
            plot(allwave[nonzero],rate)

        # save figure
        pp.savefig()


        #if the wavelenght changes start a new list
        subfiles=[]
        if fq == 'q':
            wavfile = filetype + 'q'+files[i][l1+3:l1+7]
        elif fq != 'q':
            wavfile = filetype+files[i][l1+2:l1+6]
        subfiles.append(files[i])
        #print 'files[',i,']=',files[i]
        wlen=[]
        pp.close()
